' /********************************************************** {COPYRIGHT-TOP} ****
'  * IBM Confidential
'  * Jazz for Service Management - Install
'  *
'  * (C) Copyright IBM Corp. 2013  All Rights Reserved.
'  *
'  * The source code for this program is not published or otherwise  
'  * divested of its trade secrets, irrespective of what has been 
'  * deposited with the U.S. Copyright Office.
'  ********************************************************** {COPYRIGHT-END} ***/ 

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Description: Prints to standard output or saves the current locale to a 
'              file in RFC 1766 format.
'
' Parameters:
'    - (optional) file in which to save the locale
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Option Explicit

' First argument is name of output file
Dim outputFile
If (WScript.Arguments.Count > 0) Then outputFile = WScript.Arguments.Item(0)


Dim locale, defaultLocale, tagSeparator
tagSeparator = "-"      ' RFC 1766 defines hyphen as the separator between language and region tags
defaultLocale = "en-us" ' A reasonable default in case we can not figure it out

' Try getting locale from the IBM_JAVA_OPTIONS environment variable
locale = getLocaleFromIbmJavaOptions()

' Try getting the locale from the Win32_OperatingSystem WMI object
If (locale = "") Then locale = getLocalFromWMI()

' Use default locale
If (locale = "") Then locale = defaultLocale

' If no file given as a parameter, then print on standard output
If (outputFile = "") Then
    WScript.Echo locale
Else
    Dim objFSO, objFile 
    ' Create file to write the output
    Set objFSO = WScript.createobject("Scripting.FileSystemObject")
    ' 2 => write mode; true => create file if not present
    Set objFile = objFSO.createTextFile(outputFile, 2, true)
    ' Write the obtained value and close
    objFile.WriteLine locale
    objFile.Close
End If

' All done.
WScript.quit(0)

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Build a locale string by parsing the IBM_JAVA_OPTIONS environment variable
' if it exists.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function getLocaleFromIbmJavaOptions()
    Dim objShell, objEnv, objRegEx, matches
    Dim ibmJavaOptions, returnLocale
    
    ' Get the environment variable's value
    Set objShell = WScript.createobject("WScript.Shell")
    Set objEnv = objShell.Environment("Process")
    ibmJavaOptions = objEnv("IBM_JAVA_OPTIONS")
    
    If (ibmJavaOptions <> "") Then
        'wscript.echo "IBM_JAVA_OPTIONS=" & ibmJavaOptions
        
        Set objRegEx = CreateObject("VBScript.RegExp")
        objRegEx.Global = True
        
        ' Get the user.language value
        objRegEx.Pattern = "user.language=(\w+)"
        Set matches = objRegEx.Execute(ibmJavaOptions)
        If matches.Count > 0 Then
            returnLocale = matches(0).SubMatches(0)
        End If
        
        ' Get the user.country value
        objRegEx.Pattern = "user.country=(\w+)"
        Set matches = objRegEx.Execute(ibmJavaOptions)
        If matches.Count > 0 Then
            returnLocale = returnLocale & tagSeparator & matches(0).SubMatches(0)
        End If
        
        ' Get the user.variant value
        objRegEx.Pattern = "user.variant=(\w+)"
        Set matches = objRegEx.Execute(ibmJavaOptions)
        If matches.Count > 0 Then
            returnLocale = returnLocale & tagSeparator & matches(0).SubMatches(0)
        End If
        
        Set objRegEx = Nothing
    End If
    
    ' Return what we found
    getLocaleFromIbmJavaOptions=returnLocale
End Function

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Read the locale ID from the Win32_OperatingSystem WMI object. Then lookup
' the ID in the HKEY_LOCAL_MACHINE\SOFTWARE\Classes\MIME\Database\Rfc1766 
' registry key to convert to a string
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function getLocalFromWMI() 
    const HKEY_CURRENT_USER = &H80000001
    const HKEY_LOCAL_MACHINE = &H80000002
    Dim localeId, localeName, pos
    Dim objWMI, resultSet
    
    Set objWMI = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\cimv2")
    Set resultSet = objWMI.ExecQuery("SELECT Locale FROM Win32_OperatingSystem")

    If resultSet.Count > 0 then
        localeId = resultSet.ItemIndex(0).Locale
        'WScript.Echo "Locale ID: " & localeId
        
        Set objWMI = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\default:StdRegProv")
        objWMI.GetStringValue HKEY_LOCAL_MACHINE, "SOFTWARE\Classes\MIME\Database\Rfc1766", localeId, localeName
        'WScript.Echo "Locale Name: " & localeName

        pos = InStr(localeName,";")	
        If pos > 0 then
            localeName = Left(localeName,pos-1)
            'WScript.Echo "Locale Name: " & localeName
        End If
    End If 
    
    ' Return what we found
    getLocalFromWMI = localeName
End Function


