########################################################### {COPYRIGHT-TOP} ####
# IBM Confidential
# Jazz for Service Management - Install
#
# (C) Copyright IBM Corp. 2013  All Rights Reserved.
#
# The source code for this program is not published or otherwise  
# divested of its trade secrets, irrespective of what has been 
# deposited with the U.S. Copyright Office.
########################################################### {COPYRIGHT-END} ####
#
# wsadmin jython script for creating a shared library.
# The shared library is associated with the classloader of the
# application name is being used.
# If a shared libray is found, the script exit with RC=0

import sys
import java

def getOS():
  try:
    return java.lang.System.getProperty('os.name')
  except:
    return None
###############################################################################
# definition of the getNodeId routine
#
# parameters:
#
# nodeName - the name of the node containing the server
###############################################################################
def getNodeId(nodeName):
   # make sure the node exists
   fullNodeName = "/Node:" + nodeName + "/"
   print "Looking for Node: " + fullNodeName
   nodeId = AdminConfig.getid(fullNodeName)
   if len(nodeId) == 0:
      print "   Exiting because the Node was not found"
      return None
   else:
      print "   Found Node: "  + nodeId
      return nodeId    

###############################################################################
# definition of the getServerId routine
#
# parameters:
#
# nodeName - the name of the node containing the server
# serverName - the name of the server in the node
###############################################################################
def getServerId(nodeName, serverName):
   # make sure the server exists in the node
   fullServerName = "/Node:" + nodeName + "/Server:" + serverName + "/"
   print "Looking for Server: " + fullServerName
   serverId = AdminConfig.getid(fullServerName)
   if len(serverId) == 0:
      print "   Exiting because the Server was not found"
      return
   else:
      print "   Found Server: "  + serverId
      return serverId    

###############################################################################
# definition of the getAppId routine
#
# parameters:
#
# app - the name of the app to configure 
###############################################################################
def getAppId(app):
   appId = AdminConfig.getid('/Deployment:' + app + '/');
   return appId
   
###############################################################################
# definition of the getAppId routine
#
# parameters:
#
# app - the name of the app to configure 
###############################################################################
def getWebModuleId(app,webModuleId):
   webModules = AdminConfig.list('WebModuleDeployment',app).split(lineSeparator)
   for webModule in webModules:
        warUri = AdminConfig.showAttribute(webModule, 'uri')
        print "--------------"+warUri+"---------------"
        if warUri == webModuleId:
           return webModule
 
###############################################################################
# definition of the createLibrary routine
#
# parameters:
#
# nodeId     - the id of the node containing the server
# shLibName  - the name of the shared library
# shLibDesc  - the description of the shared library
# shLibValue - the value of the shared library including the fully qualified path to the jars
#              if there are more than one jars, the user of this script is responsible for constructing
#              this value (ie, E:/ibm/tivoli/tip/coreapi.jar;E:/ibm/tivoli/tip/engineapi.jar; )
###############################################################################
def createLibrary(nodeId, shLibName, shLibDesc, shLibValue):
   print "Creating Library"
   spaceIndex=0
   try:
     spaceIndex=shLibValue.strip().index(' ')
   except:
     spaceIndex=0
   os_platform=getOS()
   if (os_platform.startswith('win') or os_platform.startswith('Win')) and spaceIndex>0:
     shLibValue = '"' + shLibValue + '"'
   libraryId = AdminConfig.create('Library', nodeId, [['name', shLibName], ['description',shLibDesc], ['classPath', shLibValue]])
   print "   Created Library: " + libraryId
   return libraryId

###############################################################################
# definition of the getClassLoaders routine
#
# parameters:
#
# appId - the application id used to retrieve the classloaders 
###############################################################################
def getClassLoaders(appId):
   print "Looking for Classloader for the Application " + appId
   classLoaderIds = AdminConfig.list('Classloader',appId).split(lineSeparator)
   return classLoaderIds

###############################################################################
# definition of the createClassLoader routine
#
# parameters:
#
# appId - the application Id used to retrieve the classloaders
# libraires - the list of librarys to add to the classloader
# mode       - the mode for the classLoader 'PARENT_LAST' or 'PARENT_FIRST'
###############################################################################
def createClassLoader(appId, libraries, mode):
   classLoaderId = AdminConfig.create('Classloader', appId, [['libraries', libraries ], ['mode', mode]])
   print "      Created Classloader: " + classLoaderId
   return classLoaderId

###############################################################################
# definition of the createLibraryRef routine
#
# parameters:
#
# classLoaderId - the id of a classloader for an application or server
# shLibName     - the name of the shared library to create a reference for
# isShared      - determines if the classloader is shared
###############################################################################
def createLibraryRef(classLoaderId, shLibName, isShared):
   print "Creating LibraryRef"
   libraryRefId = AdminConfig.create('LibraryRef', classLoaderId, [['libraryName', shLibName],  ['sharedClassloader', isShared]])
   print "   Created LibraryRef: " + libraryRefId
   return libraryRefId

###############################################################################
# definition of the add routine to add a shared library mapped to a server
#
# parameters:
# 
# nodeName   - the name of the node containing the server
# serverName - the name of the server to configure with the shared library
# shLibName  - the name of the shared library
# shLibDesc  - the description of the shared library name
# shLibValue - the value of the shared library including the fully qualified path to the jars
#              if there are more than one jars, the user of this script is responsible for constructing
#              this value (ie, E:/ibm/tivoli/tip/coreapi.jar;E:/ibm/tivoli/tip/engineapi.jar; )
# mode       - the mode for the classLoader 'PARENT_LAST' or 'PARENT_FIRST'
# isShared  - determins wheter the shLib classloader is shared 'true' or 'false' 
###############################################################################
def addShLibToServer(nodeName, serverName, shLibName, shLibDesc, shLibValue, mode, isShared):

   # variables used below
   global AdminConfig

   serverId = getServerId(nodeName,serverName)

   # make sure an application server exists for the server
   print "Looking for ApplicationServer for the Server"
   appServerId = AdminConfig.list('ApplicationServer',serverId)
   if len(appServerId) == 0:
      print "   Exiting because the ApplicationServer was not found"
      return
   else:
      print "   Found AplicationServer: " + appServerId

   #Remove now
   remove(shLibName)

   # create shared library
   #classpath = directory + LIBRARY_JAR_1 + directory + LIBRARY_JAR_2 + directory + LIBRARY_JAR_3 + directory + LIBRARY_JAR_4 + directory + LIBRARY_JAR_5 + directory + LIBRARY_JAR_6 + directory + LIBRARY_JAR_7
   print "Creating Library"
   libraryId = AdminConfig.create('Library', nodeId, [['name', shLibName], ['description',shLibDesc], ['classPath', shLibValue]])
   print "   Created Library: " + libraryId

   # look for the classloader for the application server
   print "Looking for Classloader for the ApplicationServer"
   classLoaderId = AdminConfig.list('Classloader',appServerId)

   # create the classloader if it does not exist
   if len(classLoaderId) == 0:
      print "   Did not find Classloader, creating a new Classloader"
      classLoaderId = AdminConfig.create('Classloader', appServerId, [['libraries', []], ['mode', mode]])
      print "      Created Classloader: " + classLoaderId
   else:
      print "   Found Classloader: " + classLoaderId

   # remove any existing references to the shared library
   existingLibraryRefIds = AdminConfig.list('LibraryRef',classLoaderId).split(lineSeparator)
   print "Found number of existing LibraryRefs: %s" % (len(existingLibraryRefIds))
   for existingLibraryRefId in existingLibraryRefIds:
      if len(existingLibraryRefId) > 0:
         existingLibraryRefLibraryName = AdminConfig.showAttribute(existingLibraryRefId, 'libraryName')
         print "      LibraryRef libraryName: " + existingLibraryRefLibraryName
         if existingLibraryRefLibraryName == shLibName:
            print "      Removing LibraryRef"
            AdminConfig.remove(existingLibraryRefId)
            print "      Removed LibraryRef"

   # create library ref associating appserver classloader to shared library
   print "Creating LibraryRef"
   libraryRefId = AdminConfig.create('LibraryRef', classLoaderId, [['libraryName', shLibName],  ['sharedClassloader', isShared]])
   print "   Created LibraryRef: " + libraryRefId

   # Save all the changes made by this routine
   print "Saving all configuration changes made by addSharedLib"
   AdminConfig.save()
   print "   Saved all configuration changes"

# ###############################################################################
# definition of the add routine to add a shared library mapped to a server
#
# parameters:
# 
# nodeName   - the name of the node containing the server
# appName    - the name of the J2EE application to configure with the shared library
# shLibName  - the name of the shared library
# shLibDesc  - the description of the shared library name
# shLibValue - the value of the shared library including the fully qualified path to the jars
#              if there are more than one jars, the user of this script is responsible for constructing
#              this value (ie, E:/ibm/tivoli/tip/coreapi.jar;E:/ibm/tivoli/tip/engineapi.jar; )
# mode       - the mode for the classLoader 'PARENT_LAST' or 'PARENT_FIRST'
# isShared  - determins wheter the shLib classloader is shared 'true' or 'false' 
#############################################################################################
def addShLibToApplication(nodeName, appName, shLibName, shLibDesc, shLibValue, mode, isShared):
   #get app id
   appId = getAppId(appName)

   if len(appId) == 0:
      sys.stderr.write("Exiting because the application name "+ appName +" was not found!!!\n")
      sys.exit(103)
   else:
      print "   Found Application Name: "  + appName

   # look for the classloader for the application
   classLoaderIds = getClassLoaders(appId)
    
   classLoaderId = None
   # create the classloader if it does not exist
   if len(classLoaderIds) == 0:
      print "   Did not find Classloader, creating a new Classloader"
      classLoaderId = createClassLoader(appId, [], mode)
   else:
      classLoaderId = classLoaderIds[0]
      print "   Found Classloader: " + classLoaderId

   #Check for library already existing and get out if it's there
   if ( search(shLibName) == "true"):
      print shLibName+" already exists!!\n"
      sys.exit(0)

   #Create the library
   createLibrary(getNodeId(nodeName),shLibName,shLibDesc,shLibValue)

   # create library ref associating appserver class to shared library
   print "Creating LibraryRef"
   libraryRefId = createLibraryRef(classLoaderId, shLibName, isShared)

   # Save all the changes made by this routine
   print "Saving all configuration changes made by addSharedLib"
   AdminConfig.save()
   print "   Saved all configuration changes"
   
# ###############################################################################
# definition of the update routine to add a shared library mapped to a server
#
# parameters:
# 
# nodeName   - the name of the node containing the server
# appName    - the name of the J2EE application to configure with the shared library
# shLibName  - the name of the shared library
# shLibDesc  - the description of the shared library name
# shLibValue - the value of the shared library including the fully qualified path to the jars
#              if there are more than one jars, the user of this script is responsible for constructing
#              this value (ie, E:/ibm/tivoli/tip/coreapi.jar;E:/ibm/tivoli/tip/engineapi.jar; )
# mode       - the mode for the classLoader 'PARENT_LAST' or 'PARENT_FIRST'
# isShared  - determins wheter the shLib classloader is shared 'true' or 'false' 
#############################################################################################
def updateShLibToApplication(nodeName, appName, shLibName, shLibDesc, shLibValue, mode, isShared):
   #get app id
   appId = getAppId(appName)

   if len(appId) == 0:
      sys.stderr.write("Exiting because the application name "+ appName +" was not found!!!\n")
      sys.exit(103)
   else:
      print "   Found Application Name: "  + appName

   # look for the classloader for the application
   classLoaderIds = getClassLoaders(appId)
    
   classLoaderId = None
   # create the classloader if it does not exist
   if len(classLoaderIds) == 0:
      print "   Did not find Classloader, creating a new Classloader"
      classLoaderId = createClassLoader(appId, [], mode)
   else:
      classLoaderId = classLoaderIds[0]
      print "   Found Classloader: " + classLoaderId

   #Check for library  existing and update it, else create new one
   isExist = "false"
   existingLibraryIds = AdminConfig.list('Library').split(lineSeparator)
   print "Found number of existing Libraries: %s" % (len(existingLibraryIds))
   for existingLibraryId in existingLibraryIds:
       if len(existingLibraryId) > 0:
           existingLibraryName = AdminConfig.showAttribute(existingLibraryId, 'name')
           print "      Library name: " + existingLibraryName
           if existingLibraryName == shLibName:
               isExist = "true"
               #update the library
               AdminConfig.modify(existingLibraryId, [['classPath', '']])
               AdminConfig.modify(existingLibraryId, [['classPath', shLibValue]])

   #Create the library and library reference if not exist
   if ( isExist == "false"):
        createLibrary(getNodeId(nodeName),shLibName,shLibDesc,shLibValue)
        # create library ref associating appserver class to shared library
        print "Creating LibraryRef"
        libraryRefId = createLibraryRef(classLoaderId, shLibName, isShared)

   # Save all the changes made by this routine
   print "Saving all configuration changes made by addSharedLib"
   AdminConfig.save()
   print "   Saved all configuration changes"
   
# ###############################################################################
# definition of the add routine to add a shared library mapped to a server
#
# parameters:
# 
# nodeName   - the name of the node containing the server
# appName    - the name of the J2EE application to configure with the shared library
# shLibName  - the name of the shared library
# shLibDesc  - the description of the shared library name
# shLibValue - the value of the shared library including the fully qualified path to the jars
#              if there are more than one jars, the user of this script is responsible for constructing
#              this value (ie, E:/ibm/tivoli/tip/coreapi.jar;E:/ibm/tivoli/tip/engineapi.jar; )
# mode       - the mode for the classLoader 'PARENT_LAST' or 'PARENT_FIRST'
# isShared  - determins wheter the shLib classloader is shared 'true' or 'false' 
#############################################################################################
def addShLibToWebModule(nodeName, appName, webModuleName, shLibName, shLibDesc, shLibValue, mode, isShared):
   print "--------------Enter addShLibToWebModule-------------------"
   #get app id
   appId = getAppId(appName)

   if len(appId) == 0:
      sys.stderr.write("Exiting because the application name "+ appName +" was not found!!!\n")
      sys.exit(103)
   else:
      print "   Found Application Name: "  + appName

   # look for the classloader for the application
   
   webModule = getWebModuleId(appId,webModuleName)

   print "-------"+webModule+"----------"
   
   classLoaderIds = getClassLoaders(webModule)
    
   classLoaderId = None
   # create the classloader if it does not exist
   if len(classLoaderIds) == 0:
      print "   Did not find Classloader, creating a new Classloader"
      classLoaderId = createClassLoader(webModule, [], mode)
   else:
      classLoaderId = classLoaderIds[0]
      print "   Found Classloader: " + classLoaderId

   #Check for library already existing and get out if it's there
   if ( search(shLibName) == "true"):
      print shLibName+" already exists!!\n"
      sys.exit(0)

   #Create the library
   createLibrary(getNodeId(nodeName),shLibName,shLibDesc,shLibValue)

   # create library ref associating appserver class to shared library
   print "Creating LibraryRef"
   libraryRefId = createLibraryRef(classLoaderId, shLibName, isShared)

   # Save all the changes made by this routine
   print "Saving all configuration changes made by addSharedLib"
   AdminConfig.save()
   print "   Saved all configuration changes"

###############################################################################
# definition of the remove routine
#
# parameters:
# 
# shLibName   - the name of the node shared library
# 
###############################################################################
def remove(shLibName):
   # remove any existing instances of the shared library
   existingLibraryIds = AdminConfig.list('Library').split(lineSeparator)
   print "Found number of existing Libraries: %s" % (len(existingLibraryIds))
   for existingLibraryId in existingLibraryIds:
      print "   Next Library: " + existingLibraryId
      if len(existingLibraryId) > 0:
         existingLibraryName = AdminConfig.showAttribute(existingLibraryId, 'name')
         print "      Library name: " + existingLibraryName
         print "         shLibName: " + shLibName
         if existingLibraryName == shLibName :
            print "      Removing Library"
            AdminConfig.remove(existingLibraryId)
            print "      Removed Library"
            break;
			
   existingLibraryRefIds = AdminConfig.list('LibraryRef').split(lineSeparator)
   print "Found number of existing LibrarieRefs: %s" % (len(existingLibraryRefIds))
   
   for existingLibraryRefId in existingLibraryRefIds:
      print "   Next existingLibraryRef: " + existingLibraryRefId
      if len(existingLibraryRefId) > 0:
        existingLibraryRefName = AdminConfig.showAttribute(existingLibraryRefId, 'libraryName')
        print "      LibraryRef name: " + existingLibraryRefName
        if existingLibraryRefName == shLibName :
            print "      Removing Library ref"
            AdminConfig.remove(existingLibraryRefId)
            print "      Removed Library ref"
            break;

###############################################################################
# definition of the search routine
#
# parameters:
# 
# shLibName   - the name of the node shared library
# 
###############################################################################
def search(shLibName):
    existingLibraryIds = AdminConfig.list('Library').split(lineSeparator)
    print "Found number of existing Libraries: %s" % (len(existingLibraryIds))
    for existingLibraryId in existingLibraryIds:
        if len(existingLibraryId) > 0:
            existingLibraryName = AdminConfig.showAttribute(existingLibraryId, 'name')
            print "      Library name: " + existingLibraryName
            if existingLibraryName == shLibName:
                return "true"
    return "false"


#-----------------------------------------
# Usage
#-----------------------------------------
def printUsage():
    print "Usage:  <was_profile_dir>/bin/wsadmin -lang jython -f addSharedLib.py create_MapToServer/create_MapToApplication <nodeName> <serverName> <shLibName> <shLibDesc> <shLibValue> <isShared> [success]"
    print "   where <was_profile_dir>   is your was profile directory"
    print "         <nodeName>          is the name of the node (for example, myNode01)"
    print "         <serverName|applicationName>        is the name of the server (for example, server1) or the application Name (for example, iAuthz)"
    print "         <shLibName>         is the name of the Shared Library(for example, BIRT22_SH_LIB"
    print "         <shLibDesc>         is the description of the Shared Library"
    print "         <shLibValue>        is the full path including jar files, with forward"
    print "                             slashes (/) for the file separator character,"
    print "                             and with no slash character at the end"
    print "                             (for example, E:/ibm/tivoli/tip/coreapi.jar)"
    print "         <isShared>          is this shared library shared (for example, true)"
    print "         [optional success msg] is the optional success string that will be printed if the installation went through correctly"
    print ""
    print "or <was_profile_dir>/bin/wsadmin  -lang jython -f addSharedLib.py remove [success]"

###############################################################################
# Main block -- script execution starts here
###############################################################################

# Uncomment the following line to enable tracing to the wsadmin.traceout file
# AdminControl.trace('com.ibm.*=all=enabled')

# Verify the correct number or parameters was passed in,
# then call the routine add
if (len(sys.argv) < 1):
    printUsage()
    sys.exit(101)
else:
    mode = sys.argv[0]
    print "--------------"+mode+"-------------------"
    if mode == "create_MapToApplication":
        if not (len(sys.argv) in [7,8]):
            sys.stderr.write("Invalid number of arguments\n")
            printUsage()
            sys.exit(101)
        nodeName = sys.argv[1]
        appName = sys.argv[2]
        shLibName = sys.argv[3]
        shLibDesc = sys.argv[4]
        shLibValue = sys.argv[5]
        modeSetting = "PARENT_LAST"  #Always PARENT_LAST in TIP
        isShared = sys.argv[6]
        successMsg = "Shared library installed correctly"

        if len(sys.argv) > 7:
            successMsg = sys.argv[7]


        print "Executing add function with the following parameters:"
        print "   Node: " + nodeName
        print "   Application Name : " + appName
        print "   Name: " + shLibName
        print "   Description: " + shLibDesc
        print "   Value: " + shLibValue
        print "   Parent Mode: "+modeSetting
        print "   IsShared: "+isShared

        nodeId = getNodeId(nodeName)
        if ( nodeId == None or len(nodeId) == 0 ):
            sys.stderr.write("Could not find the node\n")
            sys.exit(103)

        addShLibToApplication(nodeName,appName,shLibName,shLibDesc,shLibValue,modeSetting,isShared)
        print "Finished executing add function"

        print "Verifying the installation, if the library was installed correctly successMsg would be printed"
        if search(shLibName) == "true":
            print successMsg
    elif mode == "update_MapToApplication":
        if not (len(sys.argv) in [7,8]):
            sys.stderr.write("Invalid number of arguments\n")
            printUsage()
            sys.exit(101)
        nodeName = sys.argv[1]
        appName = sys.argv[2]
        shLibName = sys.argv[3]
        shLibDesc = sys.argv[4]
        shLibValue = sys.argv[5]
        modeSetting = "PARENT_LAST"  #Always PARENT_LAST in TIP
        isShared = sys.argv[6]
        successMsg = "Shared library installed correctly"

        if len(sys.argv) > 7:
            successMsg = sys.argv[7]


        print "Executing add function with the following parameters:"
        print "   Node: " + nodeName
        print "   Application Name : " + appName
        print "   Name: " + shLibName
        print "   Description: " + shLibDesc
        print "   Value: " + shLibValue
        print "   Parent Mode: "+modeSetting
        print "   IsShared: "+isShared

        nodeId = getNodeId(nodeName)
        if ( nodeId == None or len(nodeId) == 0 ):
            sys.stderr.write("Could not find the node\n")
            sys.exit(103)

        updateShLibToApplication(nodeName,appName,shLibName,shLibDesc,shLibValue,modeSetting,isShared)
        print "Finished executing add function"

        print "Verifying the installation, if the library was installed correctly successMsg would be printed"
        if search(shLibName) == "true":
            print successMsg
            
    elif mode == "create_MapToWebModule":
        nodeName = sys.argv[1]
        appName = sys.argv[2]
        webModuleName = sys.argv[3]
        shLibName = sys.argv[4]
        shLibDesc = sys.argv[5]
        shLibValue = sys.argv[6]
        modeSetting = "PARENT_LAST"  #Always PARENT_LAST in TIP
        isShared = sys.argv[7]
        addShLibToWebModule(nodeName,appName,webModuleName,shLibName,shLibDesc,shLibValue,modeSetting,isShared)
		
    elif mode == "create_MapToServer":
        if not (len(sys.argv) in [7,8]):
            sys.stderr.write("Invalid number of arguments\n")
            printUsage()
            sys.exit(101)
        nodeName = sys.argv[1]
        serverName = sys.argv[2]
        shLibName = sys.argv[3]
        shLibDesc = sys.argv[4]
        shLibValue = sys.argv[5]
        modeSetting = "PARENT_LAST"  #Always PARENT_LAST in TIP
        isShared = sys.argv[6]
        successMsg = "Shared library installed correctly"

        if len(sys.argv) > 7:
            successMsg = sys.argv[7]


        print "Executing add function with the following parameters:"
        print "   Node: " + nodeName
        print "   Server Name : " + serverName
        print "   Name: " + shLibName
        print "   Description: " + shLibDesc
        print "   Value: " + shLibValue
        print "   Parent Mode: "+modeSetting
        print "   IsShared: "+isShared

        nodeId = getNodeId(nodeName)
        if ( nodeId == None or len(nodeId) == 0 ):
            sys.stderr.write("Could not find the node\n")
            sys.exit(103)

        addShLibToServer(nodeName,serverName,shLibName,shLibDesc,shLibValue,modeSetting,isShared)
        print "Finished executing add function"

        print "Verifying the installation, if the library was installed correctly successMsg would be printed"
        if search(shLibName) == "true":
            print successMsg

    #This is currently not being used by TIP because we don't associate the Shared Library to a server.
    #Need to change the logics (being associate to a server) if using
    elif mode == "redeploy":
        if not (len(sys.argv) in [6,7]):
            sys.stderr.write("Invalid number of arguments\n")
            printUsage()
            sys.exit(101)
        nodeName = sys.argv[1]
        serverName = sys.argv[2]
        shLibName = sys.argv[3]
        shLibDesc = sys.argv[4]
        shLibValue = sys.argv[5]
        successMsg = "Shared library redeployed correctly"

        if len(sys.argv) > 6:
            successMsg = sys.argv[6]

        print "Removing existing "+shLibName
        remove(shLibName)

        print "Executing add function with the following parameters:"
        print "   Node: " + nodeName
        print "   Server: " + serverName
        print "   Name: " + shLibName
        print "   Description: " + shLibDesc
        print "   Value: " + shLibValue

        add(nodeName, serverName, shLibName, shLibDesc, shLibValue)
        print "Finished executing add"

        print "Verifying the installation, if the library was installed correctly successMsg would be printed"
        if search(shLibName) == "true":
            print successMsg
    elif mode == "remove":
        if not len(sys.argv) in [2,3]:
            sys.stderr.write("Invalid number of arguments\n")
            printUsage()
            sys.exit(101)
        shLibName = sys.argv[1]
        successMsg = "Shared library removed correctly"

        if len(sys.argv) > 2:
            successMsg = sys.argv[2]
        remove(shLibName)

        # Save all the changes made by this routine
        print "Saving all configuration changes made by remove function"
        AdminConfig.save()
        print "   Saved all configuration changes"

        if search(shLibName) == "false":
            print successMsg
    else:
        sys.stderr.write("Invalid command:  " + mode + "\n")
        printUsage()
        sys.exit(103)
