########################################################### {COPYRIGHT-TOP} ####
# IBM Confidential
# Jazz for Service Management - Install
#
# (C) Copyright IBM Corp. 2013  All Rights Reserved.
#
# The source code for this program is not published or otherwise  
# divested of its trade secrets, irrespective of what has been 
# deposited with the U.S. Copyright Office.
########################################################### {COPYRIGHT-END} ####

import sys

def getCellName():
    """Return the name of the cell we're connected to"""
    # AdminControl.getCell() is simpler, but only
    # available if we're connected to a running server.
    cellObjects = getObjectsOfType('Cell')  # should only be one
    cellname = getObjectAttribute(cellObjects[0], 'name')
    return cellname

def getObjectsOfType(typename, scope = None):
    """Return a python list of objectids of all objects of the given type in the given scope
    (another object ID, e.g. a node's object id to limit the response to objects in that node)
    Leaving scope default to None gets everything in the Cell with that type.
    ALWAYS RETURNS A LIST EVEN IF ONLY ONE OBJECT.
    """
    m = "getObjectsOfType:"
    if scope:
        return _splitlines(AdminConfig.list(typename, scope))
    else:
        return _splitlines(AdminConfig.list(typename))
        
def getObjectAttribute(objectid, attributename):
    """Return the value of the named attribute of the config object with the given ID.
    If there's no such attribute, returns None.
    If the attribute value looks like a list, converts it to a real python list.
    TODO: handle nested "lists"
    """
    result = AdminConfig.showAttribute(objectid, attributename)
    if result != None and result.startswith("[") and result.endswith("]"):
        # List looks like "[value1 value2 value3]"
        result = _splitlist(result)
    return result


def getNodeId( nodename ):
    return AdminConfig.getid( '/Cell:%s/Node:%s/' % ( getCellName(), nodename ) )
    
    
def getServerId(nodename,servername):
	"""Return the config id for a server or proxy.  Could be an app server or proxy server, etc"""
        id = getObjectByNodeAndName(nodename, "Server", servername) # app server
        if id == None:
            id = getObjectByNodeAndName(nodename, "ProxyServer", servername)
    	return id
    	
    	
def getObjectByNodeAndName( nodename, typename, objectname ):
    """Get the config object ID of an object based on its node, type, and name"""
    # This version of getObjectByName distinguishes by node,
    # which should disambiguate some things...
    node_id = getNodeId(nodename)
    all = _splitlines(AdminConfig.list( typename, node_id ))
    result = None
    for obj in all:
        name = AdminConfig.showAttribute( obj, 'name' )
        if name == objectname:
            if result != None:
                raise "FOUND more than one %s with name %s" % ( typename, objectname )
            result = obj
    return result
    
    
def _splitlines(s):
      rv = [s]
      if '\r' in s:
        rv = s.split('\r\n')
      elif '\n' in s:
        rv = s.split('\n')
      if rv[-1] == '':
        rv = rv[:-1]
      return rv
    

def getEndPoint( nodename, servername, endPointName):
    """Find an endpoint on a given server with the given endpoint name and return endPoint object config ID"""
    node_id = getNodeId(nodename)
    if node_id == None:
        raise "Could not find node: name=%s" % nodename
    server_id = getServerId(nodename, servername)
    if server_id == None:
        raise "Could not find server: node=%s name=%s" % (nodename,servername)
    serverEntries = _splitlines(AdminConfig.list( 'ServerEntry', node_id ))

    serverName = AdminConfig.showAttribute( server_id, "name" )

    for serverEntry in serverEntries:
        #print "serverEntry: %s" % serverEntry
        sName = AdminConfig.showAttribute( serverEntry, "serverName" )
        #print "looking at server %s" % sName
        if sName == serverName:
            specialEndPoints = AdminConfig.showAttribute( serverEntry, "specialEndpoints" )
            # that returned a string like [ep1 ep2 ep3], sigh - strip the
            # [] and split on ' '
            specialEndPoints = specialEndPoints[1:len( specialEndPoints )-1].split( " " )
            #print "specialEndPoints=%s" % repr(specialEndPoints)

            for specialEndPoint in specialEndPoints:
                endPointNm = AdminConfig.showAttribute( specialEndPoint, "endPointName" )
                #print "endPointNm=%s" % endPointNm
                if endPointNm == endPointName:
                    #print "serverEntry = %s" % serverEntry
                    return AdminConfig.showAttribute( specialEndPoint, "endPoint" )
    # Don't complain - caller might anticipate this and handle it
    #print "COULD NOT FIND END POINT '%s' on server %s" % ( endPointName, serverName )
    return None
    
    
def getServerPort( nodename, servername, endPointName ):
    """Return port number for the given endpoint.
    e.g. getServerHostnamePort(node_id, server_id, 'SIP_DEFAULTHOST')"""
    return getServerHostnamePort(nodename,servername,endPointName)[1]
    
def getServerHostName( nodename, servername, endPointName ):
    """Return port number for the given endpoint.
    e.g. getServerHostnamePort(node_id, server_id, 'SIP_DEFAULTHOST')"""
    return getServerHostnamePort(nodename,servername,endPointName)[0]
    
def getServerHostnamePort( nodename, servername, endPointName ):
    """Return [host,portnum] for the given endpoint.
    e.g. getServerHostnamePort(node_id, server_id, 'SIP_DEFAULTHOST')"""
    ePoint = getEndPoint(nodename = nodename,
                         servername = servername,
                         endPointName = endPointName )
    if ePoint == None:
        raise Exception("COULD NOT FIND END POINT %s" % endPointName)
    host = AdminConfig.showAttribute( ePoint, "host" )
    port = AdminConfig.showAttribute( ePoint, "port" )
    return ( host, port )
    
def startApplicationOnServer(appname,nodename,servername):
    """Start the named application on one servers"""
    cellname = getCellName()
    # Get the application manager MBean
    appManager = AdminControl.queryNames('cell=%s,node=%s,type=ApplicationManager,process=%s,*' %(cellname,nodename,servername))
    # start it
    AdminControl.invoke(appManager, 'startApplication', appname)
    
def stopApplicationOnServer(appname,nodename,servername):
    """Stop the named application on one servers"""
    cellname = getCellName()
    # Get the application manager MBean
    appManager = AdminControl.queryNames('cell=%s,node=%s,type=ApplicationManager,process=%s,*' %(cellname,nodename,servername))
    # stop it
    AdminControl.invoke(appManager, 'stopApplication', appname)
    
def setJVMProperties(nodename, servername, propname, propvalue):
	server_id = getServerId(nodename, servername)
	jvm = AdminConfig.list('JavaVirtualMachine', server_id)
	attrs = [["name", propname], ["value", propvalue]]
	AdminConfig.create("Property", jvm, attrs, "systemProperties")
	AdminConfig.save()
	
def removeJvmProperty(nodename, servername, propertyname):
    	server_id = getServerId(nodename, servername)
	jvm = AdminConfig.list('JavaVirtualMachine', server_id)
    	findAndRemove('Property', [['name', propertyname]], jvm)
	AdminConfig.save()
    
def findAndRemove( type, attrs, parent=None ):
	"""Removes all the objects of a specific type under a specific parent that have attributes matching attrs"""
    	children = getFilteredTypeList(type, attrs, parent)
    	for child in children:
    		AdminConfig.remove(child)
      
def getFilteredTypeList( type, attrs, parent=None ):
 	"""Produces a list of all the objects of a specific type under a specific parent that have attributes matching attrs"""
 	myList = getObjectsOfType(type, parent)
	return filterTypeList(myList, attrs)
    	
def filterTypeList( list, attrs ):
	"""Filters the input list for items with an attribute names matching the input attribute list"""
	newlist = []
	for item in list:
		# assume that the item is wanted
        	addItem = 1
        	for attr in attrs:
            		value = getObjectAttribute( item, attr[0] )
            		if value != attr[1]:
                		# if any attribute of an item is not wanted, then the item is not wanted
                		addItem = 0
        	if addItem:
            		newlist.append( item )
	return newlist