@echo off

rem ####################################################
rem #
rem # IBM Confidential
rem #
rem # (C) Copyright IBM Corp. 2012  All Rights Reserved.
rem #
rem ####################################################

set /A count=0    
for %%i in (%*) do set /A count+=1    

if not %count%==2 (
	echo Usage: %0 ^<database_name^> ^<user_name^>
	echo. 
	echo This script creates an sql script for creating IBM Cognos Content Store database.
	echo The script will be created in the current directory with the name:
	echo tcr_create_db2_cs.sql
	echo.
	echo   ^<database_name^>      - The name of the database that will be created 
	echo                          and that will be used as IBM Cognos Content Store.
	echo                          The database name can have up to 8 characters. 
	echo                          To avoid potential problems, do not use 
	echo                          special characters ^@, ^#, and ^$ in a database name.
	echo   ^<user_name^>          - The user ID used to connect to the Content Store.
	echo. 
	echo Use this syntax to run the sql script:
	echo   db2 -vtf ^<sqlscript_filename^>
	echo.
	echo To run the sql script, log in as a user with sufficient privileges to access and create the database.
	echo For the Tivoli Common Reporting to function, a database user account must exist in the system.
	echo.
	echo Example:
	echo. 
	echo The following command creates the tcr_create_db2_cs.sql script,
	echo which creates a database named tcrcontentstore in DB2 along with all the tablespaces, bufferpools,
	echo and schemas required by IBM Cognos content database.
	echo Then, the sql script grants all the required privileges for the tcruser user account.
	echo. 
	echo   TCR_create_contentstore_database_db2.bat tcrcontentstore tcruser
	echo.
	echo After running the command, you can run the generated sql script by typing the following command:
	echo   db2 -vtf tcr_create_db2_cs.sql
	exit /B 1
)

set dbName=%1
set userName=%2
set sqlScriptFileName=tcr_create_db2_cs.sql

set bufferPool08KBP=%dbName%_08KBP
set bufferPool32KBP=%dbName%_32KBP
set temporarySystemTablespace=TSN_SYS_%dbName%
set temporaryUserTablespace=TSN_USR_%dbName%
set regularTablespace=TSN_REG_%dbName%
set dbAlias=%dbName%

echo -- Licensed Materials - Property of IBM >%sqlScriptFileName%
echo -- >>%sqlScriptFileName%
echo -- BI and PM: CM >>%sqlScriptFileName%
echo -- >>%sqlScriptFileName%
echo -- (C) Copyright IBM Corp. 2010 >>%sqlScriptFileName%
echo -- >>%sqlScriptFileName%
echo -- US Government Users Restricted Rights - Use, duplication or disclosure >>%sqlScriptFileName%
echo -- restricted by GSA ADP Schedule Contract with IBM Corp. >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo -- Copyright (C) 2008 Cognos ULC, an IBM Company. All rights reserved. >>%sqlScriptFileName%
echo -- Cognos (R) is a trademark of Cognos ULC, (formerly Cognos Incorporated). >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo -- Use this script to create the IBM Cognos content database. >>%sqlScriptFileName%
echo -- %dbName% : Database name >>%sqlScriptFileName%
echo -- %userName% : User ID, this account is used by the product to connect to the content store >>%sqlScriptFileName%
echo -- This script must be run as a user that has sufficient privileges to access and create the database. >>%sqlScriptFileName%
echo -- The database user account needs to exist for the product to function. >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo CREATE DATABASE %dbName% ALIAS %dbAlias% USING CODESET UTF-8 TERRITORY US; >>%sqlScriptFileName%
echo CHANGE DATABASE %dbName% COMMENT WITH 'IBM Cognos Content Store'; >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo CONNECT TO %dbName%; >>%sqlScriptFileName%
echo UPDATE DATABASE CONFIGURATION USING APPLHEAPSZ 1024 DEFERRED; >>%sqlScriptFileName%
echo UPDATE DATABASE CONFIGURATION USING LOCKTIMEOUT 240 DEFERRED; >>%sqlScriptFileName%
echo CONNECT RESET; >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo CONNECT TO %dbName%; >>%sqlScriptFileName%
echo CREATE BUFFERPOOL %bufferPool08KBP% IMMEDIATE SIZE 1000 PAGESIZE 8K; >>%sqlScriptFileName%
echo CREATE BUFFERPOOL %bufferPool32KBP% IMMEDIATE SIZE 1000 PAGESIZE 32K; >>%sqlScriptFileName%
echo CONNECT RESET; >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo CONNECT TO %dbName%; >>%sqlScriptFileName%
echo CREATE SYSTEM TEMPORARY TABLESPACE %temporarySystemTablespace% IN DATABASE PARTITION GROUP IBMTEMPGROUP PAGESIZE 32K BUFFERPOOL %bufferPool32KBP%; >>%sqlScriptFileName%
echo CREATE USER TEMPORARY TABLESPACE %temporaryUserTablespace% IN DATABASE PARTITION GROUP IBMDEFAULTGROUP PAGESIZE 8K BUFFERPOOL %bufferPool08KBP%; >>%sqlScriptFileName%
echo CREATE REGULAR TABLESPACE %regularTablespace% IN DATABASE PARTITION GROUP IBMDEFAULTGROUP PAGESIZE 8K BUFFERPOOL %bufferPool08KBP%; >>%sqlScriptFileName%
echo CONNECT RESET; >>%sqlScriptFileName%
echo. >>%sqlScriptFileName%
echo CONNECT TO %dbName%; >>%sqlScriptFileName%
echo CREATE SCHEMA db2COGNOS AUTHORIZATION %userName%; >>%sqlScriptFileName%
echo COMMENT ON SCHEMA db2COGNOS IS 'IBM Cognos Content Store'; >>%sqlScriptFileName%
echo GRANT CREATETAB,BINDADD,CONNECT,IMPLICIT_SCHEMA ON DATABASE TO USER %userName%; >>%sqlScriptFileName%
echo GRANT CREATEIN,DROPIN,ALTERIN ON SCHEMA DB2COGNOS TO USER %userName% WITH GRANT OPTION; >>%sqlScriptFileName%
echo GRANT USE OF TABLESPACE %temporaryUserTablespace% TO USER %userName%; >>%sqlScriptFileName%
echo GRANT USE OF TABLESPACE %regularTablespace% TO USER %userName%; >>%sqlScriptFileName%
echo CONNECT RESET; >>%sqlScriptFileName%
echo The %sqlScriptFileName% script was created
